import * as log from 'loglevel';
export class MoreHelpfulParser {
    constructor(editorState) {
        this.diagnostics = new Array();
        this.tree = editorState.tree;
        this.state = editorState;
    }
    diagnose(node) {
        log.setLevel('debug');
        console.clear();
        if (node.to === 0) {
            return;
        }
        this.cursor = node.cursor;
        do {
            log.debug(this.cursor.node.type.name);
        } while (this.cursor.next());
        log.debug('\n\n');
        log.debug(node.type.name);
        this.program(this.cursor);
    }
    program(cursor) {
        console.group(cursor.node.type.name);
        cursor.next();
        log.debug(cursor.node.type.name);
        this.program_start(cursor);
        cursor.next();
        log.debug(cursor.node.type.name);
        if ('Blocks' === cursor.node.type.name) {
            this.blocks(cursor);
        }
        log.debug(cursor.node.type.name);
        this.program_end(cursor);
        log.debug(cursor.node.type.name);
        console.groupEnd();
    }
    program_start(cursor) {
        console.group(cursor.node.type.name);
        cursor.firstChild();
        log.debug(cursor.node.type.name);
        MoreHelpfulParser.consumeEOB(cursor);
        this.expect('PERCENT', cursor);
        cursor.nextSibling();
        log.debug(cursor.node.type.name);
        MoreHelpfulParser.consumeComment(cursor);
        this.end_of_block(cursor);
        MoreHelpfulParser.consumeEOB(cursor);
        log.debug(cursor.node.type.name);
        this.expect('PROGRAM_NUMBER', cursor);
        cursor.next();
        log.debug(cursor.node.type.name);
        MoreHelpfulParser.consumeComment(cursor);
        this.end_of_block(cursor);
        log.debug(cursor.node.type.name);
        // cursor.next();
        console.groupEnd();
    }
    blocks(cursor) {
        console.group(cursor.node.type.name);
        cursor.next();
        // console.log(cursor.node.type.name);
        console.groupEnd();
    }
    block(cursor) {
        console.group(cursor.node.type.name);
        console.groupEnd();
    }
    mblocks(cursor) {
        console.group(cursor.node.type.name);
        console.groupEnd();
    }
    commands(cursor) {
        console.group(cursor.node.type.name);
        console.groupEnd();
    }
    command(cursor) {
        console.group(cursor.node.type.name);
        console.groupEnd();
    }
    mcommands(cursor) {
        console.group(cursor.node.type.name);
        console.groupEnd();
    }
    end_of_block(cursor) {
        console.group(cursor.node.type.name);
        cursor.next();
        log.debug(cursor.node.type.name);
        this.expect('EOB', cursor);
        cursor.next();
        console.groupEnd();
    }
    program_end(cursor) {
        console.group(cursor.node.type.name);
        cursor.next();
        this.expect('PERCENT', cursor);
        cursor.next();
        this.expect('End_Of_Block', cursor);
        MoreHelpfulParser.consumeEOB(cursor);
        console.groupEnd();
    }
    if_statement(cursor) {
        console.group(cursor.node.type.name);
        console.groupEnd();
    }
    while_loop(cursor) {
        console.group(cursor.node.type.name);
        console.groupEnd();
    }
    do_loop(cursor) {
        console.group(cursor.node.type.name);
        console.groupEnd();
    }
    end_loop(cursor) {
        console.group(cursor.node.type.name);
        console.groupEnd();
    }
    assignment(cursor) {
        console.group(cursor.node.type.name);
        console.groupEnd();
    }
    blockskip(cursor) {
        console.group(cursor.node.type.name);
        console.groupEnd();
    }
    letter_command(cursor) {
        console.group(cursor.node.type.name);
        console.groupEnd();
    }
    goto_command(cursor) {
        console.group(cursor.node.type.name);
        console.groupEnd();
    }
    expression(cursor) {
        console.group(cursor.node.type.name);
        console.groupEnd();
    }
    subtracted_expression(cursor) {
        console.group(cursor.node.type.name);
        console.groupEnd();
        // console.log(cursor.node.type.name)
    }
    multiplied_expression(cursor) {
        console.group(cursor.node.type.name);
        console.groupEnd();
    }
    modified_atom(cursor) {
        console.group(cursor.node.type.name);
        console.groupEnd();
    }
    negate(cursor) {
        console.group(cursor.node.type.name);
        console.groupEnd();
    }
    function(cursor) {
        console.group(cursor.node.type.name);
        console.groupEnd();
    }
    atom(cursor) {
        console.group(cursor.node.type.name);
        console.groupEnd();
    }
    getDiagnostics() {
        return this.diagnostics;
    }
    addDiagnostic(cursor, msg) {
        this.diagnostics.push({
            severity: 'error',
            message: msg,
            from: cursor.from ? cursor.from : 0,
            to: cursor.to ? cursor.to : this.state.doc.length,
        });
    }
    expect(expected, cursor) {
        if (expected !== cursor.node.type.name) {
            this.addDiagnostic(cursor, 'got ' + cursor.node.type.name.toLowerCase() + ' expected ' + expected);
            // throw new Error('parse error');
        }
    }
    static consumeEOB(cursor) {
        while ('End_Of_Block' === cursor.node.type.name || 'EOB' === cursor.node.type.name) {
            console.log(cursor.node.type.name);
            cursor.next();
        }
    }
    static consumeComment(cursor) {
        while ('COMMENT' === cursor.node.type.name) {
            console.log(cursor.node.type.name);
            cursor.next();
        }
    }
}
//# sourceMappingURL=more-helpful-more-helpful-parser.js.map
