import {continuedIndent, foldNodeProp, indentNodeProp, LezerSyntax} from "@codemirror/next/syntax";
import {styleTags} from "@codemirror/next/highlight";
import {Extension} from "@codemirror/next/state";
import {SyntaxNode} from "lezer-tree";
import {parser} from "lezer-gcode";

const END_LOOP_REGEX = new RegExp('^END.*');

/// A syntax provider based on the Lezer GCODE
/// more-helpful-parser, extended with
/// highlighting and indentation information.
export const gcodeSyntax = LezerSyntax.define(parser.withProps(
    indentNodeProp.add(type => {
        if (type.name == "While_Loop") return continuedIndent({except: END_LOOP_REGEX})
        return undefined
    }),
    foldNodeProp.add({
        While_Loop(node: SyntaxNode) {
            const loopIdentifier = node.cursor.moveTo(node.to, -1);
            const endLoopLength = loopIdentifier.to - loopIdentifier.from;
            return {from: node.from + 5, to: node.to - endLoopLength}
        }
    }),
    styleTags({
        COMMENT: "comment",
        "FLOAT INT": "atom",
        PROGRAM_NUMBER: "number",
        //keywords
        "AND BCD BIN DO END GOTO IF OR THEN WHILE XOR": "keyword definition",
        "ABS ACOS ASIN ATAN COS FIX FUP ROUND SIN SQRT TAN": "keyword#2 definition",
        "LT GT LE GE NE EQ": "operatorKeyword",
        "A B C D E F G H I J K L M N O P Q R S T U V W X Y Z": "strong name#2",
        "+ - * / = #": "meta arithmeticOperator",
        "#": "meta definition",
        "( )": "meta paren",
        "[ ]": "meta bracket",
        "{ }": "meta brace"
    })
), {
    languageData: {
        commentTokens: {line: {open: "(", close: ")"}},
    },
})

/// Returns an extension that installs the gcode syntax and
/// support features.
export function gcode(): Extension {
    return [gcodeSyntax]
}
