import { gcodeSyntax } from "../gcode";
import { EditorState } from "@codemirror/next/state/dist";
function verify(s, config) {
    const diagnostics = new Array();
    let start = new Date();
    let tree;
    tree = gcodeSyntax.parser.parse(s, { strict: false });
    console.log('Parse took: ' + (new Date().getMilliseconds() - start.getMilliseconds()) + 'ms');
    // console.log(tree.toString());
    tree.iterate({
        from: 0,
        to: tree.length,
        enter: (type, start, end) => {
            // console.log(type.name, type.name.charCodeAt(0), start, end);
            if (type.name.charCodeAt(0) === 9888) {
                console.log('found error', type.name, type.name.charCodeAt(0), start, end);
                const error = new class {
                };
                error.severity = 'error';
                error.from = start;
                error.to = end;
                error.message = 'Syntax Error: Invalid character.';
                error.actions = [{
                        name: "fix",
                        apply(view, start, end) {
                            view.dispatch({
                                changes: {
                                    from: start,
                                    to: end,
                                    insert: ''
                                },
                                scrollIntoView: true
                            });
                        }
                    }];
                diagnostics.push(error);
            }
        },
        leave: () => { }
    });
    return diagnostics;
}
export function gcodeLinter(config) {
    function range(state, from = 0, to = state.doc.length) {
        return verify(state.sliceDoc(from, to), config);
    }
    return (view) => {
        let [syntax] = view.state.facet(EditorState.syntax);
        let found = [];
        // FIXME move to async parsing?
        syntax.getTree(view.state).iterate({
            enter(type, start, end) {
                if (type.name == gcodeSyntax.parser.topType.name) {
                    for (let diagnostic of range(view.state, start, end)) {
                        found.push(diagnostic);
                    }
                    return false;
                }
                // no need to iterate over the whole tree
                // if we are doing a full parse from 'program'
                return -1;
            }
        });
        return found;
    };
}
//# sourceMappingURL=gcode-linter.js.map