import { EditorState, EditorView } from '@codemirror/next/basic-setup';
import { Diagnostic } from '@codemirror/next/lint';
import { MoreHelpfulParser } from '..';
import { VagueHintParser } from '../vague-hint-parser/vague-hint-parser';
import { Parser } from '../parser/parser';

export interface LinterConfig {
    experimental: boolean;
}

export function gcodeLinter(config?: LinterConfig) {

    function range(state: EditorState, from= 0, to = state.doc.length): Diagnostic[] {
        const programText = state.sliceDoc(from, to).trimEnd();
        let parser: Parser;

        if (config.experimental) {
            parser = new MoreHelpfulParser();
        } else {
            parser = new VagueHintParser();
        }

        return parser.check(programText);
    }

    return (view: EditorView) => {
        const found: Diagnostic[] = [];
        const from = view.state.tree.topNode.from;
        const to = view.state.tree.topNode.to;
        for (const diagnostic of range(view.state, from, to)) {
            found.push(diagnostic);
        }

        return found;
    }
}
