import chai from 'chai';
import {createEditorState} from "../../test/utils";
import { VagueHintParser } from './vague-hint-parser';

describe('Program', () => {
  const tests = [
    {
      title: 'empty program',
      doc: '',
      expectedDiagnosis: []
    },
    {
      title: 'minimum program',
      doc:
        '%\n' +
        ':0000\n' +
        '%',
      expectedDiagnosis: []
    },
    {
      title: 'extra whitespace',
      doc:
        '\n' +
        '%\n' +
        '\n' +
        ':0000\n' +
        '%' +
        '',
      expectedDiagnosis: []
    },
    {
      title: 'with comment',
      doc:
        '%\n' +
        ':0000(comment)\n' +
        '%',
      expectedDiagnosis: []
    },
    {
      title: 'missing opening %',
      doc:
        '\n' +
        ':0000\n' +
        '%' +
        '',
      expectedDiagnosis: [
        {
          'from': 1,
          'message': 'from enter',
          'severity': 'error',
          'to': 6
        },
        {
          'from': 8,
          'message': 'from enter',
          'severity': 'error',
          'to': 8
        }
      ]
    },
    {
      title: 'missing closing %',
      doc:
        '%\n' +
        ':0000\n',
      expectedDiagnosis: [
        // this comes from EOB
        {
          'from': 8,
          'message': 'from leave',
          'severity': 'error',
          'to': 8
        },
        // this comes from End_Of_Block
        {
          'from': 8,
          'message': 'from leave',
          'severity': 'error',
          'to': 8
        }
      ]
    }
  ];

  tests.forEach(test => {
    const editorState = createEditorState(test.doc);
    const parser = new VagueHintParser();
    it(test.title, () => {
      let diagnostics = [];
      try {
        diagnostics = parser.check(test.doc);
      } catch (e) {}
      chai.expect(test.expectedDiagnosis).to.deep.equal(diagnostics);
    });
  });
});
