import { Diagnostic } from '@codemirror/next/lint';
import { gcodeSyntax } from '../gcode';
import { NodeType } from 'lezer-tree';
import { atEndOfProgram } from '../parser/parser-utils';
import { Parser } from '../parser/parser';
import log from 'loglevel';


export class VagueHintParser implements Parser {

  private diagnostics: Diagnostic[];

  constructor() {
    this.diagnostics = [];
  }

  public check(programText: string): Diagnostic[] {
    const diagnostics = [];

    const start = new Date();
    const tree = gcodeSyntax.parser.parse(programText, {strict: false});

    tree.iterate({
      enter(type, from, to) {
        log.debug(type.name, from, to);
        if (0 === type.id) {
          diagnostics.push({
            severity: 'error',
            message: 'from enter',
            from: from,
            to: to
          })
        }
      },
      leave(type: NodeType, from: number, to: number) {
        log.debug('leaving: ' + type.name);
        log.debug('from: ' + from);
        log.debug('to: ' + to);
        log.debug('full length: ' + programText.length);
        if (atEndOfProgram(from, programText.length)) {
          if ('PERCENT' !== type.name && 'Program_End' !== type.name) {
            diagnostics.push({
              severity: 'error',
              message: 'from leave',
              from: to,
              to: programText.length
            })
          }
        }
      }
    });

    log.debug('Parse took: ' + (new Date().getMilliseconds() - start.getMilliseconds()) + 'ms');
    return diagnostics;
  }

}
